\"
\" Trojka Core documentation
\" (c) Maarten Los
\"
\" Use the provided Makefile for generating the ps/txt file
\"
\" Here are some macros first:
\"
\" start normal list
.de sN
.KS
.DS
..
\" end normal list
.de eN
.DE
.KE
..
\" start technical list
.de sL
.KS
.DS
.ft CR
..
\" end technical list
.de eL
.DE
.KE
.ft TR
..
\" technical text
.de te
\f(CR\\$1\f(TR
..
\" horizontal line
.de hl
.br
\l'20'
..
.ft TR
.TL
System Independent Trojka Core
Version 1.0
.AU
Maarten Los
.AI
Copyright \(co 1994-1996
Last update: March 1, 1996
.bp
.SH
Copyright notice
.LP
.I
The name 'Trojka' and it's derivations 'MacTrojka' and 'XTrojka' As well
as the game concept are the intellectual property of the author and
are copyrighted.

This core engine is the standard for any trojka implementation.  Please
respect it, and stick to it when writing your own version of the game.

You are allowed to make your own implementation of the game for any
operating system you like, as long as this copyright notice is
maintained and respected. However, please do
.R
NOT
.I
redistribute implementations written for any of the platforms mentioned
hereafter, since it has already been done and also because those
versions are already managed by others.
.R

.KS
Operating systems already covered:
.TS
box center;
l l .
OS	MACHINE
_
AmigaOS	(Commodore Amiga)
MacOS 7.x	(Apple Macintosh)
MS-DOS	(PC running in text-mode)
MS-Windows 3.x	(PC)
OS/2	(PC)
Primos	(Prime Computer)
Unix	T{
(any box supporting curses)
T}
X Window System	(any box running X11R5)
.TE
.KE

.KS
Systems not yet covered (wishlist):
.TS
box center;
l l .
OS	MACHINE
_
Archimedes	(Acorn)
Atari	(Atari)
VMS	(VAX)
Windows '95	(PC)
MS DOS	(VGA version)
.TE
.KE

.SH
Distribution policy
.LP
This core is freeware. Permission to copy, use, and distribute this
software and its documentation is granted, as long as you don't change the
copyright notice in any of the parts of the package.
 
Newly implemented Trojka versions should be freeware. You are not
allowed to distribute a commercial version of it without permission of
the author.

.bp
.nr A \n%
.NH S 1
INTRODUCTION
.LP
This manual is used to describe
.I Trojka.
It covers the ins and outs of the game, gives tips on what
a Trojka implementation
.I might
do and what it 
.I should
do. We assume you already know how to play
Trojka and make multi-million scores...

This core is meant to make an easy implementation possible on as many
platforms supporting C as possible. It is an attempt to bring uniformity
in the scoring and internal operation. It's also easy for anyone who
knows about programming to make an implemenation.

This documentation is divided into another 4 chapters. Chapter 2 handles
about the 'look & feel' of Trojka. Things become more technical after chapter
3, in which the operation of the core is described. Chapter 4 explains the
application programmers interface to the core, and chapter 5 describes
the calls that the core makes to the user part.


.bp
.nr B \n%
.NH S 2
TROJKA LOOK AND FEEL
.NH 2
Dimensions
.LP
A Trojka playing field is 20 blocks high and its width is 5 blocks.
All Trojka blocks should have a width:height ratio of 2:1, which
means a width:height ratio of 1:2 for the playing field.
.NH 2
Block colors
.LP
Trojka uses blocks of five different colors. It was difficult to find
five real distinct colors, but this seems to be the best choice:
.sN
	light green
	light blue
	dark blue
	light magenta
	light brown\/ yellow
.eN

.LP
The colors are best visible on a black background. On monochrome screens
patterns must be used.

For graphic screens, the following patterns seem to be best. They are
also used in the monochrome X version of the game. Only the basic
patterns are given, which must be used as a fill style. A 'O' is a black
pixel, a '.' is a white pixel.
.sL
block 1 (2x2 pattern)
	O.
	.O

block2  (2x2 pattern)
	O.
	O.

block3 (4x4 pattern)
	.O..
	O...
	..O.
	...O

block4 (4x4 pattern)
	OO..
	O..O
	..OO
	.OO.

block5 (1x1 pattern)
	O
	
.eL

They look good when displayed on a black background, but the inverted patterns
look good on a white background.

When in monochrome text mode, the following set built up of ASCII characters can
be used.
.sL
block 1
 	####
block 2
	OOOO
block 3
	||||
block 4
	$$$$
block 5
	%%%%
.eL
.bp
.nr C \n%
.NH S 3
CORE OVERVIEW
.NH 2
The\ core
.LP
This core is meant for those who want to write an implementation of
Trojka for a new operating system except the ones mentioned in the
copyright notice. The
.I core
provides a standard engine around which
Trojka can be built. It features things like scanning, keeping track of
game speed and score, managing the wizard mode and its layout system.
The core consists of the following parts:
.KS
.DS C
TROJKA API
ENGINE
USER API
.DE
.KE
.LP
The
.I Trojka\ API
is set of function calls and global variables meant to
interact with the engine. The
.I engine
handles the game itself. The
.I User\ API
is a collection of empty function calls to be implemented by the
programmer. These parts together form the Trojka Core. Around it is the
system dependent 
.I shell.


.NH 2
The shell
.LP
The Trojka implementation built around the engine using API calls is
called the
.I shell.
The shell communicates with the engine using the Trojka API calls.
The engine
does some interesting things and if needed calls the User API in turn. How to
write a shell, which functions you should call and which are the global
variables is described in chapter 4 and 5.

You can also make calls directly to the engine, but it is not recommended
unless you know what you're doing. Also, you take the risk of messing
up the score system and the speed system which takes us to the following:
this core wasn't just written merely for fun. It is also an attempt to
synchronize
the scoring and speeding across several platforms. That is why
.I you\ should\ not
tamper with the score, speed and other variables, they ought to be read only.

.NH 2
What the core does 
.LP
The core takes care of:
.ID
\(bu  moving the block according to input.
\(bu  scanning the playing field
\(bu  redrawing the playing field
\(bu  managing the wizard mode
\(bu  managing the score
\(bu  counting the number of total dropped block so far
\(bu  counting the number of each seperate type of block
\(bu  counting the number of trojkas
\(bu  counting the number of blocks wiped
\(bu  managing the speed
.DE

.NH 2
What the core does NOT

.LP
The core does
.I not

.ID
\(bu  manage the highscore list
\(bu  draw the score 
\(bu  draw fancy things on the screen
\(bu  handle menus
\(bu  redraw windows
\(bu  play music
\(bu  all other funny things.
.DE

.bp
.nr D \n%
.NH S 4
THE TROJKA API
.LP
This chapter gives an overview of all the variables and the API-calls
that have to be used. It is reference to be used when reading chapter 5
about the User API and the shell.

The data types, constants and variables are defined in 
.te tr_core.h.

.NH 2
Data Types
.LP
The Core uses the following data types:
.KS
.TS
l l .
Name	Type
_
tt_bool	boolean
tt_int	integer
tt_long	unsigned long
tt_command	see below
.TE
.KE

.NH 2
Trojka constants
.LP
These are read-only constants.
.KS
.TS
l l .
Name	Description
_
tc_pm_top	The top of the playing matrix
tc_pm_bottom	The bottom of the playing matrix
tc_blocks	The number of different blocks.
.TE
.KE

These are commands to be passed within the
.te tt_command
structure discussed in a following paragraph. 

.KS
.ID
	tc_c_init
	tc_c_blockdown
	tc_c_blockleft
	tc_c_blockright
	tc_c_dropblock
	tc_c_speedup
	tc_c_setspeed
	tc_c_drawfield
.KE
.DE

.LP
These are result values of the 
.te tc_c_blockdown
command.
.KS
.ID
	tc_res_normal
	tc_res_gameover
	tc_res_touchdown
.KE
.DE
.NH 2
External variables
.LP
Listed here are the external variables. 
The
.te xxx_reset
variables are all true when the variable
.te xxx
has reached its maximum. This figure lies around 4 billion (the size of an
unsigned long) so they are virtually never used. They are available for
those who need them. Variables are all
.I read\ only.
The only way to control things should be through the API calls.

.KS
.TS
l l l .
Datatype	Name	Description
_
tt_int	tv_ticks	T{
The delay time in milliseconds before the block is moved down.
T}
tt_long	tv_score	T{
The current score.
T}
tt_bool	tv_score_reset	T{
True if the score has reached its maximum.
T}
tt_long	tv_blocks	T{
The number of fallen blocks.
T}
tt_bool	tv_blocks_reset	T{
True if tv_blocks > maximum.
T}
tt_long	tv_trojkas	T{
Number of Trojkas.
T}
tt_bool	tv_trojkas_reset	T{
True if tv_trojkas > maximum.
T}
tt_long	tv_wipes	T{
Number of wipes.
T}
tt_long	tv_wipes_reset	T{
True if tv_wipes > maximum.
T}
tt_long	tv_block_count	T{
Array containing the number of blocks for each block color.
T}
tt_int	tv_speed	T{
The current game speed.
T}
tt_bool	tv_is_wizard	T{
True if the wizard mode is turned on.
T}
.TE
.KE


.NH 2
Trojka API calls
.LP
The Trojka Core is to be called only by the following call:
.sL
	trojka_api(command);
.eL
.LP
where
.te command
is the address of the following structure:
.sL
	typedef struct _tt_command {
		tt_int command;
		tt_long param1;
		tt_long param2;
	} tt_command;		/* command for the trojka api */
.eL

.LP
Listed on the following pages are the commands and their parameters:
.hl
.IP COMMAND
.te tc_c_init
.IP PARAM1
.te (tt_int)initial\ speed
.IP PARAM2
.te (tt_bool)wizard
.IP SYNOPSIS
This should be the first call before each new game is
started. Initializes the Trojka Core for a game with the
initial speed given in
.te param1.
.te Param2
defines if the
game is to be played in wizard mode. Later on, wizard mode
can be turned on by the 
.te tc_c_setwizard
command. Resets
.I all
external variables to their initial value.
.bp
.IP CALLBACKS
-	
.IP RETURNS
-

.hl
.IP COMMAND
.te tc_c_drawfield
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
Draws the playing field and if in wizard mode, also draws a
layout on it. This call should be given after 
.te tc_c_init.
.IP CALLBACKS
.te trojka_make_block_callback
.IP RETURNS
	-

.hl
.IP COMMAND
.te tc_c_blockdown
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
Moves the block down one position and redraws it. It must be
called iteratively with a pause of 
.te tv_ticks
millisecs
between the calls. If the block hits the bottom or another
block, it scans the playing field and redraws it when there
are wipes. Also check for trojkas and spiders. If the bottom is
reached again after a succesful scan.
.IP CALLBACKS
.sL
	trojka_make_block_callback
	trojka_wipe_block_callback
	trojka_explode_callback (if needed)
	trojka_speedup_callback (if needed)
	trojka_trojka_callback (if needed)
	trojka_spider_callback (if needed)
	trojka_bottom_callback (if needed)
.eL
.IP RETURNS
.RS
.IP tc_res_touchdown
if the block plainly hit the ground or another block.
In this case scanning has been done.
.IP tc_res_normal
the block could move down normally
.IP tc_res_gameover
the top of the screen is reached and the game is over
.RE

.hl
.IP COMMAND
.te tc_c_blockleft
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
If possible, it moves the block left one position and
redraws it. Should be called on pressing a left cursor key.
.IP CALLBACKS
.sL
	trojka_make_block_callback
	trojka_wipe_block_callback
.eL
.IP RETURNS
-

.hl
.IP COMMAND
.te tc_c_blockright
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
If possible, it moves the block to the right one position
and redraws it. Should be called on pressing a right cursor
key.
.IP CALLBACKS
.sL
	trojka_make_block_callback
	trojka_wipe_block_callback
.eL
.IP RETURNS
-

.hl
.IP COMMAND
.te tc_c_dropblock
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
Drops the block down until it hits another block or the
bottom. A next call to 
.te tc_c_blockdown
detects the block
has hit the bottom and acts accordingly, see description of
this function.  CALLBACKS trojka_make_block_callback
.IP CALLBACKS
.sL
	trojka_make_block_callback
	trojka_wipe_block_callback
.eL
.IP RETURNS
-

.hl
.IP COMMAND
.te tc_c_speedup
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
Gives a request to the enige to speed up the game forcedly.
This call should ONLY be made on the user pressing the key
you defined for 'speedup'. The speed cannot exceed the
value of '9'.
.IP CALLBACKS
.te trojka_speedup_callback
.IP RETURNS
-

.hl
.IP COMMAND
.te tc_c_setspeed
.IP PARAM1
.te (tt_int)speed
.IP PARAM2
-
.IP SYNOPSIS
Similar to
.te tc_c_speedup
but now used to directly set the
game on the speed given. The speed is set to the closest
value between 1 and 9 of that given in the parameter.
.IP CALLBACKS
.te trojka_speedup_callback
.IP RETURNS
-

.hl
.IP COMMAND
.te tc_c_setwizard
.IP PARAM1
-
.IP PARAM2
-
.IP SYNOPSIS
Turn on the wizard mode. This is only useful when playing
the game, if the game was not initialized in this mode.
.IP CALLBACKS
-
.IP RETURNS
-
.bp
.nr E \n%
.NH S 5
THE USER API AND THE SHELL
.LP
This chapter gives an overview the user API and how to write a shell;
a Trojka implementation around the engine.

The Trojka engine regularly calls functions that have to be defined by
the programmer, since these are either very system and implemenation
dependent or are essential for game interaction. Here they are:

.NH 2
User API calls
.SH
.te void\ trojka_make_block_callback(tt_int\ x,\ tt_int\ y,\ tt_int\ color);
.LP
This function should draw a block of color or pattern 
.te color
on the screen at virtual coordinates
.te x
and
.te y
To understand the system of virtual see the paragraph about writing a
Trojka implementation.

.SH
.te void\ trojka_wipe_block_callback(tt_int\ x,\ tt_int\ y);
.LP
Draw a blank block at virtual coordinates
.te x
and
.te y.
This routine should erase any visible block at the given coordinates.

.SH
.te void\ trojka_explode_callback(tt_int\ x,\ tt_int\ y);
.LP
This function is called each time a block is wiped. You could program it
to make a fancy sound or draw an explosion on the screen. The parameters
.te x
and
.te y
give a hint about where the explosion might be drawn. The
engine takes care of wiping the blocks and redrawing the field.

.SH
.te void\ trojka_trojka_callback(tt_long\ trojka_score);
.LP
This function is called when a trojka is made.
.te trojka_score
contains
the score that is rewarded for it, which is usually 3333, but is doubled
for each next trojka in row (this seldom happens...). You can use this
figure to display somewhere or not use it at all. The engine takes care
of adding it to the score as usual.

.SH
.te void\ trojka_spider_callback(tt_long\ spider_score);
.LP
When a 'spider' is made this function is called.
.te Spider_score
contains the rewarded score which you can use for displaying it somewhere. 
The engine takes care of adding it to the score.

.SH
.te void\ trojka_bottom_callback(tt_long\ bottom_score);
.LP
This function is called when the bottom of the playing field is reached
after one or more wipes. The rewarded score is passed in 
.te bottom_score.
The engine takes care of adding it to the score.

.SH
.te void\ trojka_speedup_callback(tt_int\ speed);
.LP
This function is called when the speed is increased by the engine. The
current speed is passed in the 
.te speed
variable. You can also retrieve
it using the global variable
.te tv_speed.

.NH 2
Writing a Trojka implementation
.LP
Now a sample is given on how to make a Trojka implementation. Only the
most relevant variables and functions are given, the others like
.te show_score()
are left to the imagination of the programmer.


First of all, include the core header file.
.sL
	.
	.
	#include "tr_core.h"
	.
	.
.eL

.LP
It is useful to pre-define a set of regularly used commands.
.sL
	.
	.
	tc_command command;
	tc_command cmd_draw = { tc_c_drawfield, 0, 0 };
	tc_command cmd_down = { tc_c_down, 0, 0 };
	tc_command cmd_left = { tc_c_left, 0, 0 };
	tc_command cmd_right = { tc_c_right, 0, 0 };
	tc_command cmd_drop = { tc_c_dropblock, 0, 0 };
	.	
	.
.eL

.LP
The result of the
.te tc_c_down
call.
.sL
	.
	.
	tt_int 	down_res;
	.
	.
.eL
.LP
Since all external engine variables should be read only, you'll need the
following local variables in your own system dependent shell.  You can
give 
.te speed
a starting value you like, but 5 is the most common.
.te wizard
can be toggled on/off by the user when not playing, but it
shouldn't be possible to turn it off when playing.
.sL
	.
	.
	tt_int	speed;
	tt_bool	wizard;
	.
	.
.eL
.LP
The first thing you do is initialize the game and draw the playing field.
.sL
	.
	.
	command.command = tc_c_init;
	command.param1 = speed;
	command.param2 = wizard;

	trojka_api(&command);
	trojka_api(&cmd_draw);
	.
	.
.eL
.LP
Now the game can be started. A typical game loop looks like this. In any
way, make sure that a block is moved down on the
.te tv_ticks
interval.
.sL
	.
	.
	gameover = false;

	while(!gameover) {

		down_res = trojka_api(&cmd_down);
		draw_score( tv_score );

		if(down_res == tc_c_gameover)
			gameover = true;

		key = get_keyboard_code();

		switch(key) {

			case KEY_MOVE_LEFT:
				trojka_api(&cmd_left);
				break;

			case KEY_MOVE_RIGHT:
				trojka_api(&cmd_right);	
				break;
			
			case KEY_DROP_BLOCK:
				trojka_api(&cmd_dropblock);
				break;
			
			case KEY_SPEEDUP:
				trojka_api(&cmd_speedup);
				speed = tv_speed;
				break;

			case KEY_SWITCH_TO_WIZARD:
				trojka_api(&cmd_setwizard);
				wizard = tv_is_wizard;
				break;
		}
			
		delay_milliseconds(tv_ticks);		
	}
	.
	.
.eL
.LP
Note that only the most important frame is given. Other things like
showing the statistics based on
.te tv_blocks
and
.te tv_blockcount
can also be implemented.
.LP
The programmer should also take care of the correct implemenations of
the callbacks. The most important of them are the ones who deal with
virtual coordinates; the 
.te drawblock,
.te wipeblock
and 
.te explode
callbacks. To understand the virtual coordinates, you'll have to
understand how the Trojka engine stores its playing matrix. It is stored
like this:
.so /tmp/picture
.LP
The y-coordinates that are passed are in the range from
.I tc_pm_top
to
.I tc_pm_bottom,
that's from 20 
.I downto
1. The x-coordinates that are
passed go from 0 up to 4. The virtual coordinates have to be translated
to that on your own drawing field.

As you can read in the chapter about 'Look and feel', all
blocks have to have a width:height ratio of 2:1, so let's assume that
you have a drawing area of 100 pixels wide. Having five blocks next
to eachother, this means that every block
is 20 pixels wide. With a 2:1 ratio the height is 10 pixels, which makes
the total height of the drawing area 10*20 = 200. Of course things may
vary, but in any case the width:height ratio should be 2:1. With this
information the 'drawblock' callback routines may look like this:
.sL
	.
	.
	void trojka_drawblock_callback(x, y, color)
	tt_int x,y,color;
	{
		SetColor(color);
		DrawRectangle(x, y, x * 20, y * 10);
	}

	void trojka_wipe_block_callback(x, y)
	tt_int x,y;
	{
		SetColor(color_black);	
		DrawRectangle(x, y, x * 20, y * 10);

		/*
		 * or you can do it more cleverly....
		 */
		trojka_drawblock_callback(x, y, color_black);
	}

	void trojka_explode_callback(x, y)
	tt_int x, y;
	{
		int i;

		for(i = 0; i < 100; i++) {
			trojka_drawblock_callback(color_black);
			delay_milliseconds(2);
			trojka_drawblock_callback(color_black);
			delay_milliseconds(2);
		}
	}
.eL
.LP
The other callbacks may be empty or filled in as desired.
.bp
.nr F \n%
.NH S 6
HISTORY
.LP
If you find this uninteresting you can skip it now :-).

The first version of Trojka was written in 1989 using Pascal on a PC.

When I went to the highschool the code was still in Pascal and was
ported to work under Primos on TVI-950 terminals connected to the Prime
mainframe. Among with the other games and programs collected in the
shared 'WORKDIR' it soon became popular.  This version couldn't have
seen the light without the inspiring help of Joran Jesssurun and Richard
Moeskops who also wrote a version for the Commodore Amiga. A visual basic
Windoze version was soon available, this version was written by Jac
Goudsmit. 

Alas, the ol'
Prime has long gone and is replaced by PC's and a thing called Novell.

At the Young Scientist Eindhoven, it was finally rewritten to C, to make
it run on a Xenix/68000 system. The terminal version distribution is
still based on it, and yes I know, it can use a major upgrade.

When I bought my first computer, a Macintosh Classic, the first thing I
did was get Trojka running on it. It took weeks and finally run and it
even runs on color Macs (wow...) thanks to using the Performa 400 (also 
known LCII) of Eric Toonen on which we spent many hours of hacking.

As time went by, the Xenix system at the Young Scientists was replaced
by a Sun386i based Unix-network, with graphical workstations running X11R5.
The version written for this system ran for about a year, before it
was released on the internet.

For now, a lot of version are around, at which point I decided to standardize
it. I hope that if you ever plan to write a version for your system, you
stick to the document and the core, because it standardizes a lot of things.

.bp
.SH
INDEX
.LP
.KS
.TS
center;
l r .
INTRODUCTION	\nA
TROJKA LOOK AND FEEL	\nB
CORE OVERVIEW	\nC
THE TROJKA API	\nD
THE USER API AND THE SHELL	\nE
HISTORY	\nF
.TE
.KE
